@@ -8,7 +8,7 @@ from astropy.modeling.fitting import LevMarLSQFitter
 from photutils import CircularAperture, CircularAnnulus
 from .. import FitsManager
 from photutils.psf import IntegratedGaussianPRF, DAOGroup, BasicPSFPhotometry
-from ..core import Block
+from .. import Block
 from ..utils import register_args
 try:
     import sep
@@ -51,8 +51,31 @@ class PhotutilsPSFPhotometry(Block):
 
 
 class PhotutilsAperturePhotometry(Block):
-    """
-    Aperture photometry using :code:`photutils`.
+    r"""
+    Aperture photometry using the :code:`CircularAperture` and :code:`CircularAnnulus` of photutils_ with a wide range of apertures. By default annulus goes from 5 fwhm to 8 fwhm and apertures from 0.1 to 10 times the fwhm with 0.25 steps (leading to 40 apertures).
+
+    The error (e.g. in ADU) is then computed following:
+
+    .. math::
+    
+        \sigma = \sqrt{S + (A_p + \frac{A_p}{A_n})(b + r^2 + \frac{gain^2}{2}) + scint }
+
+
+    .. image:: images/aperture_phot.png
+        :align: center
+        :width: 110px
+
+    with :math:`S` the flux (ADU) within an aperture of area :math:`A_p`, :math:`b` the background flux (ADU) within an annulus of area :math:`A_n`, :math:`r` the read-noise (ADU) and :math:`scint` is a scintillation term expressed as:
+
+
+    .. math::
+
+        scint = \frac{S_fd^{2/3} airmass^{7/4} h}{16T}
+
+    with :math:`S_f` a scintillation factor, :math:`d` the aperture diameter (m), :math:`h` the altitude (m) and :math:`T` the exposure time.
+
+    The positions of individual stars are taken from :code:`Image.stars_coords` so one of the detection block should be used, placed before this one.
+
     For more details check https://photutils.readthedocs.io/en/stable/aperture.html
 
     |write| 
@@ -204,13 +227,33 @@ class PhotutilsAperturePhotometry(Block):
 
 
 class SEAperturePhotometry(Block):
-    """
-    Aperture photometry using :code:`sep`.
-    For more details check https://sep.readthedocs.io
+    r"""
+    OUTDATED - TODO
+
+    Aperture photometry using `sep <https://sep.readthedocs.io>`_, a python wrapper around the C Source Extractor.
+
+    The error (e.g. in ADU) is then computed following:
+
+    .. math::
+
+        \sigma = \sqrt{S + (A_p + \frac{A_p}{A_n})(b + r^2 + \frac{gain^2}{2}) + scint }
+
+
+    .. image:: images/aperture_phot.png
+        :align: center
+        :width: 110px
+
+    with :math:`S` the flux (ADU) within an aperture of area :math:`A_p`, :math:`b` the background flux (ADU) within an annulus of area :math:`A_n`, :math:`r` the read-noise (ADU) and :math:`scint` is a scintillation term expressed as:
+
+
+    .. math::
+
+        scint = \frac{S_fd^{2/3} airmass^{7/4} h}{16T}
+
+    with :math:`S_f` a scintillation factor, :math:`d` the aperture diameter (m), :math:`h` the altitude (m) and :math:`T` the exposure time.
+
+    The positions of individual stars are taken from :code:`Image.stars_coords` so one of the detection block should be used, placed before this one.
 
-    SEP is a python wrapping of the C Source Extractor code, hence being 2 times faster that Photutils version.
-    Forced aperture photometry can be done simple by using :code:`stack=True` on the detection Block used, hence using
-    stack sources positions along the photometric extraction.
 
     Parameters
     ----------
