@@ -1,10 +1,9 @@
 import astroalign
 import numpy as np
 from scipy.spatial import KDTree
-from ..twirl import find_transform
-from ..twirl import utils as tutils
+from twirl import utils as tutils
 from skimage.transform import AffineTransform as skAT
-from ..core import Block
+from ..block import Block
 from ..utils import register_args
 
 def distance(p1, p2):
@@ -26,10 +25,8 @@ def clean_stars_positions(positions, tolerance=50, output_id=False):
     )
     for i, _distances in enumerate(distance_to_others):
         _distances[i] = np.inf
-        close_stars = np.argwhere(_distances < tolerance)
-        if len(close_stars) > 0:
-            keep.append(np.min([close_stars[0][0], i]))
-        else:
+        close_stars = np.flatnonzero(_distances < tolerance)
+        if len(close_stars) == 0:
             keep.append(i)
 
     if output_id:
@@ -208,8 +205,9 @@ class DistanceRegistration(Registration):
 
 
 class XYShift(Registration):
-    """
-    Compute the linear shift between two point clouds. Star coordinates in the image are expected in image.stars_coords
+    r"""Compute the linear shift between two point clouds. Star coordinates in the image are expected in image.stars_coords
+
+    |write|  ``Image.dx``, ``Image.dy``, ``Image.header`` 
 
     Parameters
     ----------
@@ -219,6 +217,47 @@ class XYShift(Registration):
         maximum difference for which shifts are considered similar, by default 1.5
     clean : bool, optional
         Merge coordinates if too close, by default False
+
+
+    Notes
+    -----
+    **xyshift** is originaly the **TRAPHOT** (M. Gillon) method to align consecutive images.
+
+    *Principle*: Let's consider two images, ``im`` and ``ref``. We want to know the shift between ``im`` and ``ref`` knowing the position of the stars in these two images. We suppose that there is N stars in ``im`` and ``ref`` and that their position are given by
+
+    .. math::
+
+        \boldsymbol{S^{im}} = \begin{bmatrix}
+        x^{im}_0 & y^{im}_0 \\
+        x^{im}_1 & y^{im}_1 \\
+        . & . \\
+        x^{im}_N & y^{im}_N
+        \end{bmatrix} \quad and \quad 
+        \boldsymbol{S^{ref}} = \begin{bmatrix}
+        x^{ref}_0 & y^{ref}_0 \\
+        x^{ref}_1 & y^{ref}_1 \\
+        . & . \\
+        x^{ref}_N & y^{ref}_N
+        \end{bmatrix}
+
+
+    :math:`x_i, y_i` being the coordinates of the star :math:`i`
+
+    If we want to know, let's say, the :math:`x` shift between `im` and `ref` we just have to compute :math:`x^{im}_0 - x^{ref}_0`, or to be more accurate, by considering all the stars, the mean shift
+
+    .. math::
+
+        \Delta x = \frac{1}{N}\sum_{0}^{N} x^{im}_i - x^{ref}_i
+
+
+    This is possible because :math:`x^{im}_i`  and :math:`x^{ref}_i` are the position of the same identified star :math:`i` (:math:`N` of them in each image)
+
+    Identifying stars in the sky, for example using their position with respect to constellations or by trying to match catalogs (as with astrometry.net) is possible with a single image, but is time-consuming when applied to a complete set of images. Unfortunately most of the time detected stars do not hold the same index :math:`i` (index is attributed by the star detection algorithm) and their number might differ.
+
+    The goal of **xyshift** is to check all the possible shifts from one star with respect to all the others, and to identify the most common shifts between stars of two different images.
+
+    .. image:: images/xyshift_principle.png
+
     """
 
     @register_args
@@ -242,6 +281,8 @@ class AstroAlignShift(Registration):
     """
     Compute the linear shift between point clouds using :code:`astroalign`
 
+    |write| ``Image.dx``, ``Image.dy``,``Image.header``
+
     `astroalign <https://astroalign.readthedocs.io/en/latest/>`_ is a python module used to align stellar astronomical images using 3-point asterisms (triangles) similarities. For speed, reference asterisms are computed once at the begining of the reduction and then matched with every images.
     """
 
@@ -290,7 +331,7 @@ class _Twirl(Block):
         self.n = n
 
     def run(self, image, **kwargs):
-        x = find_transform(image.stars_coords, self.ref, n=self.n)
+        x = tutils.find_transform(image.stars_coords, self.ref, n=self.n)
         image.transform = skAT(x)
         image.dx, image.dy = image.transform.translation
         image.header["TWROT"] = image.transform.rotation
@@ -305,6 +346,8 @@ class Twirl(Block):
     """
     Affine transform computation for images registration
 
+    |write| ``Image.dx``, ``Image.dy``,``Image.header``, ``Image.transform``
+
     Parameters
     ----------
     ref : (2, n) np.ndarray
@@ -315,15 +358,6 @@ class Twirl(Block):
 
     @register_args
     def __init__(self, ref, n=10, **kwargs):
-        """[summary]
-
-        Parameters
-        ----------
-        ref : [type]
-            [description]
-        n : int, optional
-            [description], by default 10
-        """
         super().__init__(**kwargs)
         self.ref = ref[0:n]
         self.n = n
