@@ -7,12 +7,25 @@ import xarray as xr
 from itertools import product
 from tqdm import tqdm
 from . import models
+from . import viz
+from .console_utils import info
 
 
 def nu(n, sw, sr):
     return (sw ** 2) / n + sr ** 2
 
 
+def append_tex_order(current, n, name):
+    _str = r""
+    if n == 0:
+        return current
+    elif n == 1:
+        _str += name
+    elif n > 1:
+        _str += fr"{name}^{n}"
+    current.append(_str)
+
+
 def pont2006(x, y, n=30, plot=True, return_error=False):
     dt = np.median(np.diff(x))
     ns = np.arange(1, n)
@@ -40,6 +53,7 @@ def binned_std(fluxes, bins=12):
 
 
 def diff(fluxes, errors=None, weights=None, comps=None, alc=False):
+    
     # not to divide flux by itself
     sub = np.expand_dims((~np.eye(fluxes.shape[-2]).astype(bool)).astype(int), 0)
 
@@ -48,23 +62,28 @@ def diff(fluxes, errors=None, weights=None, comps=None, alc=False):
         weights = np.zeros(fluxes.shape[0:-1])
         weights[np.arange(fluxes.shape[0]), comps.T] = 1.
 
+    # Formulae
+    # https://en.wikipedia.org/wiki/Weighted_arithmetic_mean#Standard_error
+
     weighted_fluxes = fluxes * np.expand_dims(weights, -1)
     art_lc = (sub @ weighted_fluxes) / np.expand_dims(weights @ sub[0], -1)
     lcs = fluxes / art_lc
 
-    returns = lcs
+    if not alc and errors is None:
+        return lcs
 
     if errors is not None:
-        weighted_errors = errors * np.expand_dims(weights, -1)
-        squarred_art_error = (sub @ weighted_errors) ** 2 / np.expand_dims(weights @ sub[0], -1)
+        weighted_errors = errors**2 * np.expand_dims(weights, -1)**2
+        squarred_art_error = (sub @ weighted_errors) / np.expand_dims(weights**2 @ sub[0], -1)
         lcs_errors = np.sqrt(errors ** 2 + squarred_art_error)
         returns = [lcs, lcs_errors]
 
-    if alc:
-        returns.append(art_lc)
-
-    return returns
-
+        if alc:
+            return [lcs, lcs_errors, art_lc]
+        else:
+            return [lcs, lcs_errors]
+    else:
+        return [lcs, art_lc]
 
 def broeg(fluxes, tolerance=1e-2, max_iteration=200, bins=12):
 
@@ -73,7 +92,7 @@ def broeg(fluxes, tolerance=1e-2, max_iteration=200, bins=12):
     idxs = np.arange(n * bins)
 
     def error_estimate(f):
-        return np.array(np.split(f.take(idxs, axis=-1), n, axis=-1)).std(-1).mean(0)
+        return np.nanmean(np.nanstd(np.array(np.split(f.take(idxs, axis=-1), n, axis=-1)), axis=-1), axis=0)
 
     i = 0
     evolution = 1e25
@@ -91,8 +110,10 @@ def broeg(fluxes, tolerance=1e-2, max_iteration=200, bins=12):
             std = error_estimate(lcs)
             weights = 1 / std
 
+        weights[~np.isfinite(weights)] = 0
+
         # Keep track of weights
-        evolution = np.std(np.abs(np.mean(weights, axis=-1) - np.mean(last_weights, axis=-1)))
+        evolution = np.nanstd(np.abs(np.nanmean(weights, axis=-1) - np.nanmean(last_weights, axis=-1)))
 
         last_weights = weights
         lcs = diff(fluxes, weights=weights)
@@ -136,6 +157,29 @@ def best_stars(fluxes, weights, target, return_idxs=True, bins=12):
         return _weights
 
 
+def scargle(time, flux, error, periods, X=None, n=1):
+
+    if X is None:
+        X = np.atleast_2d(np.ones_like(time))
+
+    variability = lambda p: models.harmonics(time, p, n)
+
+    chi2 = []
+
+    for p in tqdm(periods):
+        x = models.design_matrix([X.T, *variability(p)])
+        residuals = x @ np.linalg.lstsq(x, flux, rcond=None)[0] - flux
+        chi2.append(-np.sum((residuals / error) ** 2))
+
+    chi2 = np.array(chi2)
+
+    return chi2
+
+    # idxs = np.argwhere(np.abs(chi2) < 30*np.std(chi2)).flatten()
+    # periods = periods[idxs]
+    # chi2 = chi2[idxs]
+
+
 class ApertureFluxes:
 
     def __init__(self, xarray):
@@ -157,7 +201,7 @@ class ApertureFluxes:
 
     @property
     def target(self):
-        return self.xarray.target
+        return self.xarray.attrs['target']
 
     @target.setter
     def target(self, value):
@@ -184,7 +228,7 @@ class ApertureFluxes:
 
     def mask(self, mask, dim="time"):
         new_self = self.copy()
-        new_self.xarray = new_self.xarray.where(xr.DataArray(mask, dims=dim), drop=True)
+        new_self.xarray = self.x.sel(**{dim: self.x[dim][mask]})
         return new_self
 
     @staticmethod
@@ -238,12 +282,13 @@ class ApertureFluxes:
     def x(self):
         return self.xarray
 
+    @property
+    def has_diff(self):
+        return hasattr(self, "diff_flux")
+
     @property
     def diff_flux(self):
-        if self.corrected_diff_flux is not None:
-            return self.corrected_diff_flux
-        else:
-            return self.xarray.diff_fluxes.isel(apertures=self.aperture, star=self.target).values
+        return self.xarray.diff_fluxes.isel(apertures=self.aperture, star=self.target).values
 
     @property
     def raw_flux(self):
@@ -258,23 +303,34 @@ class ApertureFluxes:
         return self.xarray.raw_errors.isel(apertures=self.aperture, star=self.target).values
 
     @property
-    def corrected_diff_flux(self):
-        if "corrected_diff_flux" in self:
-            return self.xarray["corrected_diff_flux"].values
+    def detrended_diff_flux(self):
+        if "trends" in self:
+            return self.diff_flux - self.trend + 1
         else:
             return None
 
-    @corrected_diff_flux.setter
-    def corrected_diff_flux(self, value):
-        if value is not None:
-            self.xarray["corrected_diff_flux"] = ("time", value)
-        elif "corrected_diff_flux" in self:
-            self.xarray = self.xarray.drop("corrected_diff_flux")
+    @property
+    def trend(self):
+        if "trends" in self:
+            return self.xarray.trends[self.target].values
+        else:
+            return None
 
     @property
     def comparison_raw_fluxes(self):
         return self.raw_fluxes[self.aperture, self.comps[self.aperture]]
 
+    @property
+    def X(self):
+        if "polynomial_trend_orders" in self.xarray.attrs:
+            orders = list(zip(
+                self.xarray.attrs["polynomial_trend_variables"],
+                self.xarray.attrs["polynomial_trend_orders"])
+            )
+            return self.polynomial(**dict(orders))
+        else:
+            return None
+
     def __copy__(self):
         return self.__class__(self.xarray.copy())
 
@@ -384,7 +440,7 @@ class ApertureFluxes:
         if not inplace:
             return new_self
 
-    def broeg2005(self, inplace=True, cut=True, target=False):
+    def broeg2005(self, inplace=True, cut=None, nans=False):
         """
         The Broeg et al. 2005 differential photometry algorithm
 
@@ -394,10 +450,13 @@ class ApertureFluxes:
         ----------
         inplace: bool, optional
             whether to perform the changes on current Observation or to return a new one, default True
-        cut: bool, optional
-            whether to pick the best comparison stars and apply unitary weights, default True
+        cut: bool or None, optional
+            whether to pick the best comparison stars and apply unitary weights, default None which try both and
+            cut if beneficial
+        nans: bool
+            whether to keep nans values in fluxes, otherwise set to 1 (arbitrarly low value) before computing the weights, default False, i.e. removing nans
         """
-
+        # TODO: ignore apertures out of the image
         if inplace:
             new_self = self
         else:
@@ -407,27 +466,53 @@ class ApertureFluxes:
         raw_fluxes = self.raw_fluxes.copy()
         raw_errors = self.raw_errors.copy()
 
+        if not nans:
+            mask = np.isnan(raw_fluxes)
+            raw_fluxes[mask] = 1
+            raw_errors[mask] = 1
+
         mean_raw_fluxes = np.expand_dims(raw_fluxes.mean(-1), -1)
         raw_errors /= mean_raw_fluxes
         raw_fluxes /= mean_raw_fluxes
 
         # finding weights
         weights = broeg(raw_fluxes)
-        if not target:
-            weights[:, self.target] = 0
+        weights[:, self.target] = 0
 
-        # best comparisons
-        if cut:
-            comparisons = best_stars(raw_fluxes, weights, self.target)
-            weights = np.ones_like(comparisons)
-            diff_fluxes, diff_errors, alcs = diff(raw_fluxes, raw_errors, comps=comparisons, alc=True)
-        else:
-            # this only works for 3D fluxes (with apertures)
-            comparisons = np.repeat(np.expand_dims(np.arange(raw_fluxes.shape[-2]), -1).T, raw_fluxes.shape[0], axis=0)
-            diff_fluxes, diff_errors, alcs = diff(raw_fluxes, raw_errors, weights=weights, alc=True)
-            if not target:
-                comparisons = np.delete(comparisons, self.target, axis=1)
-                weights = np.delete(weights, self.target, axis=1)
+        comparisons = best_stars(raw_fluxes, weights, self.target)
+
+        # We always try with and without a cut
+        # -----————---------------------------
+        # - with
+        cut_comparisons = best_stars(raw_fluxes, weights, self.target)
+        cut_weights = np.ones_like(comparisons)
+        cut_diff_fluxes, cut_diff_errors, cut_alcs = diff(raw_fluxes, raw_errors, comps=comparisons, alc=True)
+
+        # - without
+        comparisons = np.repeat(np.expand_dims(np.arange(raw_fluxes.shape[-2]), -1).T, raw_fluxes.shape[0], axis=0)
+        diff_fluxes, diff_errors, alcs = diff(raw_fluxes, raw_errors, weights=weights, alc=True)
+        comparisons = np.delete(comparisons, self.target, axis=1)
+        weights = np.delete(weights, self.target, axis=1)
+
+        # cutting or not
+        if cut is True:
+            diff_fluxes, diff_errors, alcs = cut_diff_fluxes, cut_diff_errors, cut_alcs
+            comparisons, weights = cut_comparisons, cut_weights
+        elif cut is None:
+            # we decide if cutting it beneficial
+            bins = np.min([self.time.shape[-1], 12])
+            b = self.time.shape[-1] // bins
+            idxs = np.arange(b * bins)
+
+            def error_estimate(f):
+                return np.array(np.split(f.take(idxs, axis=-1), b, axis=-1)).std(-1).mean(0)
+
+            cut_std = error_estimate(cut_diff_fluxes[:, self.target]).min()
+            uncut_std = error_estimate(diff_fluxes[:, self.target]).min()
+
+            if cut_std < uncut_std:
+                diff_fluxes, diff_errors, alcs = cut_diff_fluxes, cut_diff_errors, cut_alcs
+                comparisons, weights = cut_comparisons, cut_weights
 
         # setting xarray
         dims = self.xarray.raw_fluxes.dims
@@ -461,12 +546,29 @@ class ApertureFluxes:
     # Plotting
     # ========
 
-    def plot(self, which="None", bins=0.005, color="k", std=True):
-        binned = self.binn(bins, std=std)
-        plt.plot(self.time, self.diff_flux, ".", c="gainsboro", zorder=0, alpha=0.6)
-        plt.errorbar(binned.time, binned.diff_flux, yerr=binned.diff_error, fmt=".", zorder=1, color=color, alpha=0.8)
+    def plot(self, star=None, bins=0.005, color="k", std=True, mask=True):
+        if star is None:
+            star = self.target
+        viz.plot(self.time, self.x.diff_fluxes[self.aperture, star], std=std, bins=bins, bincolor=color)
 
-    def sigma_clip(self, sigma=3.):
+    def plot_detrended(self, star=None, bins=0.005, color="k", std=True, fancy=False, ylim=None, label=True):
+        if star is None:
+            star = self.target
+        diff_flux = self.diff_fluxes[self.aperture, star]
+        trend = self.trends[star]
+
+        if fancy:
+            self.plot_systematics_signal(trend, signal=None, ylim=ylim)
+            if label:
+                viz.corner_text(f"{self.trend}", c="C0", loc=(0.05, 0.03))
+        else:
+            viz.plot(self.time, diff_flux - trend + 1, std=std, bins=bins, bincolor=color)
+            if label:
+                viz.corner_text(f"detrended ({self.trend})", c="C0")
+            if ylim is not None:
+                plt.ylim(ylim)
+                
+    def sigma_clip(self, sigma=3., star=None):
         """Sigma clipping
 
         Parameters
@@ -474,20 +576,29 @@ class ApertureFluxes:
         sigma : float, optional
             sigma clipping threshold, by default 3.
 
+        star : int, optional
+            star on which to apply the sigma clip, is target by default
+
         """
         new_self = self.copy()
-        new_self.xarray = new_self.xarray.sel(
-            time=self.time[np.abs(self.diff_flux - np.median(self.diff_flux)) < sigma * np.std(self.diff_flux)])
+        if star is None:
+            new_self.xarray = new_self.xarray.sel(
+                time=self.time[np.abs(self.diff_flux - np.median(self.diff_flux)) < sigma * np.std(self.diff_flux)])
+        else:
+            new_self.xarray = new_self.xarray.sel(
+                time=self.time[np.abs(self.diff_fluxes[self.aperture, star] - np.median(
+                    self.diff_fluxes[self.aperture, star])) < sigma * np.std(self.diff_fluxes[self.aperture, star])])
+
         return new_self
 
     # modeling
 
-    def trend(self, dm, split=None):
+    def lstsq(self, X, star=None, split=None):
         """Given a design matrix return the fitted trend
 
         Parameters
         ----------
-        dm : np.ndarray
+        X : np.ndarray
             design matrix of shape (time, n), n being the number of regressors.
         split : int or array, optional
             splitting indexes of the design matrix, passed to np.split and used to retrieve splitted models, by default None
@@ -497,15 +608,18 @@ class ApertureFluxes:
         [type]
             [description]
         """
-        w, dw, _, _ = np.linalg.lstsq(dm, self.diff_flux, rcond=None)
+        if star is None:
+            star = self.target
+
+        w, dw, _, _ = np.linalg.lstsq(X, self.diff_fluxes[self.aperture, star], rcond=None)
         if split is not None:
             if not isinstance(split, list):
                 split = [split]
             split_w = np.split(w, [-1])
-            split_dm_T = np.split(dm.T, [-1])
-            return [_w@_dm for _w, _dm in zip(split_w, split_dm_T)]
+            split_dm_T = np.split(X.T, [-1])
+            return [_w @ _dm for _w, _dm in zip(split_w, split_dm_T)]
         else:
-            return dm @ w
+            return X @ w
 
     def polynomial(self, **orders):
         """Return a design matrix representing a polynomial model
@@ -556,7 +670,7 @@ class ApertureFluxes:
 
     def dm_ll(self, dm):
         n = len(self.time)
-        chi2 = (self.diff_flux - self.trend(dm)) ** 2
+        chi2 = (self.diff_flux - self.lstsq(dm)) ** 2
         return np.sum(-(n / 2) * np.log(2 * np.pi * self.diff_error ** 2) - (1 / (2 * (self.diff_error ** 2))) * chi2)
 
     def dm_bic(self, dm):
@@ -572,7 +686,7 @@ class ApertureFluxes:
         add : np.ndarray, optional
             additional regressor to add to the design matrix, by default None
         verbose : bool, optional
-            wether to show the progress bar, by default False
+            whether to show the progress bar, by default False
         """
         def progress(x):
             return tqdm(x) if verbose else x
@@ -589,6 +703,40 @@ class ApertureFluxes:
         bics = [self.dm_bic(dm) for dm in progress(dms)]
         return dms_dicts[np.argmin(bics)]
 
+    def polynomial_trend(self, bic=True, verbose=True, **kwargs):
+        if kwargs == dict() and bic:
+            n = 3
+            kwargs = dict(sky=n, dx=n, dy=n, airmass=n, fwhm=n)
+
+        if bic:
+            orders = self.best_polynomial(**kwargs)
+        else:
+            orders = kwargs
+
+        # compute trend for all stars
+        X = self.polynomial(**orders)
+        trends = np.array([self.lstsq(X, star=s) for s in self.star])
+        self.xarray["trends"] = (("stars", "time"), trends)
+
+        # Display and save trend model
+        polynomial_tex = [r"1"]
+        original_orders = orders.copy()
+        orders = list(orders.items())
+        orders = sorted(orders, key=lambda x: x[1])
+
+        for variable, order in orders:
+            append_tex_order(polynomial_tex, order, variable)
+        polynomial_tex = rf"${'+'.join(polynomial_tex)}$"
+
+        if verbose:
+            info(f"Polynomial trend:")
+            viz.print_tex(polynomial_tex)
+
+        self.xarray.attrs["trend"] = polynomial_tex
+        self.xarray.attrs["trend_model"] = "polynomial"
+        self.xarray.attrs["polynomial_trend_variables"] = list([o[0] for o in orders])
+        self.xarray.attrs["polynomial_trend_orders"] = list([o[1] for o in orders])
+
     def noise_stats(self, bins=0.005, verbose=True):
         pont_w, pont_r = self.pont2006(plot=False)
         binned = self.binn(bins, std=True)
