@@ -36,14 +36,6 @@ def design_matrix(model_list):
     return dm
 
 
-try:
-    import theano
-    import theano.tensor as tt
-    theano.config.compute_test_value = "warn"
-except ModuleNotFoundError:
-    pass
-
-
 class LinearModel:
 
     def __init__(self, design_matrix):
@@ -65,3 +57,5 @@ class LinearModel:
         coeffs = reduce(tt.dot, [self.U.T, ty.T, 1.0 / self.S_, self.V])
         return tt.dot(coeffs, self.tX.T)
 
+
+
