@@ -1,15 +1,18 @@
-from astropy.time import Time
-from datetime import datetime, timedelta
+from datetime import timedelta
 import numpy as np
 from astropy.visualization import ZScaleInterval
 from astropy.io import fits
 import numba
 import astropy.constants as c
 import urllib
+from astropy.time import Time
+from astropy.table import Table
+from datetime import datetime
+import inspect
+from scipy import ndimage
 
 earth2sun = (c.R_earth / c.R_sun).value
 
-
 def remove_sip(dict_like):
 
     for kw in [
@@ -218,7 +221,7 @@ def check_class(_class, base, default):
     elif isinstance(_class, base):
         return _class
     else:
-        raise TypeError("ubclass of {} expected".format(base.__name__))
+        raise TypeError("subclass of {} expected".format(base.__name__))
 
 
 def divisors(n):
@@ -310,6 +313,68 @@ def jd_to_bjd(jd, ra, dec):
 def remove_arrays(d):
     copy = d.copy()
     for name, value in d.items():
-        if isinstance(value, list):
+        if isinstance(value, (list, np.ndarray)):
             del copy[name]
-    return copy
\ No newline at end of file
+    return copy
+
+
+def sigma_clip(y, sigma=5., return_mask=False, x=None):
+    mask = np.abs(y - np.nanmedian(y)) < sigma * np.nanstd(y)
+
+    if return_mask:
+        return mask
+
+    else:
+        if x is None:
+            return y[mask]
+        else:
+            return x[mask], y[mask]
+
+
+def register_args(f):
+    """
+    When used within a class, saves args and kwargs passed to a function
+    (mostly used to record __init__ inputs)
+    """
+    def inner(*args, **kwargs):
+        self = args[0]
+        self.args = args[1::]
+        self.kwargs = kwargs
+        return f(*args, **kwargs)
+    return f
+    
+
+def nan_gaussian_filter(data, sigma=1., truncate=4.):
+    """https://stackoverflow.com/questions/18697532/gaussian-filtering-a-image-with-nan-in-python
+
+    Parameters
+    ----------
+    U : _type_
+        _description_
+    sigma : _type_, optional
+        _description_, by default 1.
+    truncate : _type_, optional
+        _description_, by default 4.
+    """
+
+    V=data.copy()
+    V[np.isnan(data)]=0
+    VV=ndimage.gaussian_filter(V,sigma=sigma,truncate=truncate)
+
+    W=0*data.copy()+1
+    W[np.isnan(data)]=0
+    WW=ndimage.gaussian_filter(W,sigma=sigma,truncate=truncate)
+
+    return VV/WW
+
+def clean_header(header_dict):
+    return {key: value for key, value in header_dict.items() if not isinstance(value, (list, tuple)) and key.isupper()}
+
+
+def easy_median(images):
+    # To avoid memory errors, we split the median computation in 50
+    images = np.array(images)
+    shape_divisors = divisors(images.shape[1])
+    n = shape_divisors[np.argmin(np.abs(50 - shape_divisors))]
+    return np.concatenate([np.nanmedian(im, axis=0) for im in np.split(images, n, axis=1)])
+
