@@ -12,6 +12,7 @@ import matplotlib.pyplot as plt
 from skimage.transform import resize
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 from matplotlib.ticker import AutoMinorLocator
+from matplotlib.legend_handler import HandlerPatch
 import matplotlib.patches as mpatches
 from .models import transit
 
@@ -122,7 +123,21 @@ class AnchoredHScaleBar(matplotlib.offsetbox.AnchoredOffsetbox):
                                                         **kwargs)
 
 
-def plot_lcs(data, w=4, show=None, hide=None, ylim=None, size=None, labels=None, bins=0.005, force_width=True):
+def multiplot(
+        data,
+        bins=0.005,
+        std=True,
+        color="gainsboro",
+        bincolor="k",
+        alpha=0.6,
+        binalpha=0.8,
+        w=4,
+        show=None,
+        hide=None,
+        ylim=None,
+        size=None,
+        labels=None,
+        force_width=True):
     """Plot multiple x, y with some shared axis
 
     Parameters
@@ -148,7 +163,7 @@ def plot_lcs(data, w=4, show=None, hide=None, ylim=None, size=None, labels=None,
     """
 
     if size is None:
-        size = (4,3)
+        size = (4, 3)
     
     if isinstance(data[0], dict):
         data = [[d["time"], d["lc"]] for d in data]
@@ -158,23 +173,23 @@ def plot_lcs(data, w=4, show=None, hide=None, ylim=None, size=None, labels=None,
     if hide is None:
         hide = []
     
-    idxs = np.setdiff1d(show, hide)
+    indexes = np.setdiff1d(show, hide)
 
-    H = np.ceil(len(idxs) / w).astype(int)
+    H = np.ceil(len(indexes) / w).astype(int)
     if not force_width:
-        w = np.min([len(idxs), w])
+        w = np.min([len(indexes), w])
     fig, axes = plt.subplots(H, w, figsize=(w * size[0], H * size[1]))
+    fig.patch.set_facecolor('white')
+    max_duration = np.max([jd.max() - jd.min() for jd, _ in [data[i] for i in indexes]])
     
-    max_duration = np.max([jd.max() - jd.min() for jd, _ in [data[i] for i in idxs]])
-    
-    for _i, ax in enumerate(axes.flat if len(idxs) > 1 else [axes]):
-        if _i < len(idxs):
-            i = idxs[_i]
+    for _i, ax in enumerate(axes.flat if len(indexes) > 1 else [axes]):
+        if _i < len(indexes):
+            i = indexes[_i]
             plt.sca(ax)
             jd, lc = data[i]
             center = jd.min() + (jd.max() - jd.min())/2
 
-            plot(jd, lc, bins=bins)
+            plot(jd, lc, bins=bins, std=std, bincolor=bincolor, color=color, alpha=alpha, binalpha=binalpha)
             if ylim is not None:
                 plt.ylim(ylim)
 
@@ -194,7 +209,7 @@ def plot_lcs(data, w=4, show=None, hide=None, ylim=None, size=None, labels=None,
                     textcoords="axes fraction",
                     ha="left",
                     va="bottom",
-                    fontsize=10,
+                    fontsize=12,
                     color="k",
                 )
         else:
@@ -326,7 +341,7 @@ def show_stars(image, stars=None, highlight=None, size=15, options={}, flip=None
         plt.tight_layout()
 
 
-def plot_marks(x, y, label=None, position="bottom", offset=7, fontsize=12, color=[0.51, 0.86, 1.], ms=12, n=None, inside=True, alpha=1):
+def plot_marks(x, y, label=None, position="bottom", offset=7, fontsize=12, color=[0.51, 0.86, 1.], ms=12, n=None, inside=True, alpha=1, ax=None):
     y_offset = ms + offset
 
     if position == "top":
@@ -335,11 +350,22 @@ def plot_marks(x, y, label=None, position="bottom", offset=7, fontsize=12, color
     if not isinstance(x, (list, np.ndarray, tuple)):
         x = np.array([x])
         y = np.array([y])
-        if label is not None:
+        if label is True:
+            label = np.array([0])
+        elif label is not None:
             label = np.array([label])
+    else:
+        if label is True:
+            label = np.arange(len(x))
+        elif label is not None:
+            label = np.array(label)
+
+
+    if ax is None:
+        ax = plt.gcf().axes[0]
 
     if inside:
-        ax = plt.gcf().axes[0]
+        ax = ax
         xlim, ylim = np.array(ax.get_xlim()), np.array(ax.get_ylim())
         xlim.sort()
         ylim.sort()
@@ -347,7 +373,8 @@ def plot_marks(x, y, label=None, position="bottom", offset=7, fontsize=12, color
         x = x[within]
         y = y[within]
         if label is not None:
-            label = np.array(label)[within]
+            print
+            label = label[within]
 
     if n is not None:
         x = x[0:n]
@@ -365,7 +392,7 @@ def plot_marks(x, y, label=None, position="bottom", offset=7, fontsize=12, color
         f = 5
         if _label is not None:
             plt.annotate(_label, xy=[_x, _y - y_offset], color=color, ha='center', fontsize=fontsize, alpha=alpha,
-                         va="top" if position is "bottom" else "bottom")
+                         va="top" if position == "bottom" else "bottom")
 
 
 def fancy_show_stars(
@@ -576,7 +603,7 @@ def fancy_gif_image_array(image, median_psf, factor=0.25):
     return np.fromstring(canvas.tostring_rgb(), dtype='uint8').reshape(height, width, 3)
 
 
-def TeX(a, fmt='{: 0.3f}', dim=True):
+def array_to_tex(a, fmt='{: 0.3f}', dim=True):
     """Display a LaTeX matrix in notebook
 
     Parameters
@@ -619,6 +646,11 @@ def TeX(a, fmt='{: 0.3f}', dim=True):
     display(Math(r"{}".format(teX_math)))
 
 
+def print_tex(tex):
+    from IPython.display import display, Math
+    display(Math(r"{}".format(tex)))
+
+
 def plot_expected_transit(time, epoch, period, duration, depth=None, color="gainsboro"):
     tmax = time.max()
     t_epoch = tmax - epoch
@@ -729,7 +761,7 @@ def polynomial_trend_latex(**kwargs):
     return rf"${' + '.join(monomials)}$"
 
 
-def corner_text(text, loc=(0.05, 0.05), va="bottom", ha='left', fontsize=12):
+def corner_text(text, loc=(0.05, 0.05), va="bottom", ha='left', fontsize=12, c="k", ax=None):
     """Plot a text on the corner of axe
 
     Parameters
@@ -745,11 +777,12 @@ def corner_text(text, loc=(0.05, 0.05), va="bottom", ha='left', fontsize=12):
     fontsize : int, optional
          as in plt.txt, by default 12
     """
-    ax = plt.gca()
-    plt.text(*loc, text, fontsize=fontsize, ha=ha, va=va, transform=ax.transAxes)
+    if ax is None:
+        ax = plt.gca()
+    ax.text(*loc, text, fontsize=fontsize, ha=ha, va=va, transform=ax.transAxes, color=c)
 
 
-def plot_systematics_signal(x, y, systematics, signal, ylim=None, offset=None, figsize=(6, 7), signal_label=None):
+def plot_systematics_signal(x, y, systematics, signal=None, ylim=None, offset=None, figsize=(6, 7), signal_label=None):
     """Plot a systematics and signal model over data. systeamtics + signal is plotted on top, signal alone on detrended 
     data on bottom
 
@@ -768,21 +801,56 @@ def plot_systematics_signal(x, y, systematics, signal, ylim=None, offset=None, f
     signal_label : str, optional
         label of signal, by default None
     """
-    
-    amplitude = np.percentile(y, 95) - np.percentile(y, 5)
-    amplitude *= 1.5
-    if offset is None:
+    if ylim is not None:
+        amplitude = ylim[1] - ylim[0]
+        offset = amplitude
+        ylim = (ylim[0] - amplitude, ylim[1])
+    else:
+        amplitude = np.percentile(y, 95) - np.percentile(y, 5)
+        amplitude *= 1.5
         offset = amplitude
     if ylim is None:
         ylim = (1 - offset - 0.9 * amplitude, 1 + 0.9 * amplitude)
+    if signal is None:
+        signal = np.zeros_like(x)
+        has_signal = False
+    else:
+        has_signal = True
 
     fig = plt.figure(figsize=figsize)
     fig.patch.set_facecolor('white')
     plot(x, y, label='data', binlabel='binned data (7.2 min)')
     plt.plot(x, systematics + signal, c="C0",
-             label="systematics + signal model")
-    plt.plot(x, signal + 1. - offset, label=signal_label, c="k")
+             label=f"systematics {'+ signal' if has_signal else ''} model")
+    if has_signal:
+        plt.plot(x, signal + 1. - offset, label=signal_label, c="k")
     plt.text(plt.xlim()[1] + 0.005, 1, "RAW", rotation=270, va="center")
     plot(x, y - systematics + 1. - offset)
     plt.text(plt.xlim()[1] + 0.005, 1 - offset, "DETRENDED", rotation=270, va="center")
-    plt.ylim(ylim)
\ No newline at end of file
+    plt.ylim(ylim)
+
+
+class HandlerEllipse(HandlerPatch):
+    def create_artists(self, legend, orig_handle,
+                       xdescent, ydescent, width, height, fontsize, trans):
+        center = 0.5 * width - 0.5 * xdescent, 0.5 * height - 0.5 * ydescent
+        p = mpatches.Ellipse(xy=center, width=height + xdescent,
+                             height=height + ydescent)
+        self.update_prop(p, orig_handle, legend)
+        p.set_transform(trans)
+        return [p]
+
+
+def circles_legend(colors, texts):
+    c = [mpatches.Circle((0.5, 0.5), radius = 0.25, fill=None, ec=colors[i]) for i in range(len(texts))]
+    plt.legend(c, texts , bbox_to_anchor=(1, 1.05), loc='upper right', ncol=3, handler_map={mpatches.Circle: HandlerEllipse()}, frameon=False)
+
+
+def plot_signal(x, y, label=None, **kwargs):
+    axes = plt.gcf().axes
+    for i, ax in enumerate(axes):
+        xmin, xmax = ax.get_xlim()
+        idxs = (xmin <= x) & (x <= xmax)
+        ax.plot(x[idxs], y[idxs], label=label if i == 0 else None, **kwargs)
+        if label is not None and i == 0:
+            ax.legend()
\ No newline at end of file
